;;; --------------------------------------------------------------------------;
;;; ISOTHREADS.LSP - By David Kitson. 
;;;   (C) Copyright David Kitson. 
;;; version 0.5 ALPHA
;;;  
;;;   Based on the old THREADS routing I wrote, this vertical thread routing
;;;   does some nice stuff. It draws cross-sectional logitudinal profiles of 
;;;   a half thread, repeats several times until it's create a half-tube and
;;;   then LOFT's them together ( press OK for each LOFT command )
;;;   And then completes the thread for either internal (NUT) or external 
;;;   (BOLT) style ISO conforming threads.
;;;
;;;   These routines are primarily intended for Prototyping Purposes - ie
;;;   3D printing, Laser Sintering and such...
;;;   If you're doing that kind of work and need custom modifications, please
;;;   let me know - davidk@dropbeartech.com
;;;   New ideas for this application help everyone :) 
;;;
;;; --------------------------------------------------------------------------;
;;; DESCRIPTION
;;;
;;;   Designed and implemented by David Grant Kitson in December 2013 when he
;;;	discovered that CorelCAD had a faulty SWEEP function and couldn't cut
;;; 	threads in the traditional way. So he decided to write his own in LISP.
;;;
;;;   Yes, writing LISP without experience is a pretty crazy idea. But no one
;;;     was going to do it for me... 
;;;
;;;   This program constructs a THREAD SIDE PROFILE. It can be loaded and 
;;;   called by typing either "BOLT" for external threads or "NUT" for internal
;;;   threads. 
;;;
;;;   It currently assumed 32 segments per "circle" - because it divides up
;;;	nicely. Loft will smooth them out anyway... It's very accurate. 
;;;
;;; --------------------------------------------------------------------------;

(defun myerror (s)                    ; If an error (such as CTRL-C) occurs
                                      ; while this command is active...
  (if (/= s "Function cancelled")
    (princ (strcat "\nError: " s))
  )
  (setvar "cmdecho" ocmd)             ; Restore saved modes
  (setvar "blipmode" oblp)
  (setq *error* olderr)               ; Restore old *error* handler
  (princ)
)


;;; Inside thread first... Because I don't really know what I'm doing yet.
;;; So later will call other names - eg, othread, mithread, mothread
;;; Should produce an ISO conforming standard metric thread.

(defun connect ( sections )
  (command "_loft" sections)
  )

(defun boltaxis (start end)
  (command "_polyline3d" start)
  (command end)
  (command "")
  )

;;; ISO standard profile BOLT pitch generation routine. 
;;;
(defun cbolt (basepoint diameter pitch steppos numthreads / segments rads rinc id segmentangle faceinc facedist nextpoint startpoint nextpoint dinv vinc vstep xyspace loop) 
  (setvar "blipmode" 0)                         ; turn blipmode off
  (setvar "cmdecho" 0)                          ; turn cmdecho off
  (setq segments 32)		                ; 32 segments in this circle.
  (setq rads (* 3.141592653 2))                 ; PI x 2 ( Radians in a circle )
  (setq rinc (/ rads segments))                 ; Angle Increment in Radians = 1/Segment of a circle.
  (setq id (- diameter (* 0.75775 pitch)))       ; Internal Diameter = 7/8 of Thread Height ( ISO standard )
  (setq segmentangle (* rads (/ steppos pitch))); Segmentangle = 0 ( starting at origin... ) or twist more as progress...
  (setq faceinc (*  pitch (/ 1.732 segments)))  ; 60 degrees step per variation. 
  (setq facedist (/ diameter 2))	        ; initial distance of face from center = radius.

  (setq basepoint (list (car basepoint) (cadr basepoint) (- (+ (caddr basepoint) steppos) pitch))) ; Update base for step position and start one turn early. 
  (setq basepoint (polar basepoint segmentangle (- facedist (* pitch 0.8660)))) ; And offset most of the way... - PITCH.
  (setq facedist (* pitch 0.8660)) ; adjusted new face distance. 

  (setq startpoint (polar basepoint segmentangle facedist)) ; Set the last point to the same as the first point. 
  (command "3dpoly" basepoint startpoint)       ; Draw the bottom line of a vertical thread.
  (setq vinc 0.0)				; Vertical change in line.
  (setq dinc 0.0)				; Distance from basepoint change in line.
  (setq vstep (* 0.03125 pitch))
  (setq nextpoint startpoint)
;;; Now ready to start drawing vertical line.
;;; Four x rinc upwards

  
;;; Number of threads and re-calculate offset to remove first thread later.
;;;  this loops enough for all thread rises, plus 2 thread rises for the start and end.
  (setq loop (+ (/ length pitch) 2))
  (while (> loop 0) (setq loop (- loop 1))
  

  (setq vinc (* vstep 4))
  (setq nextpoint (list (car nextpoint) (cadr nextpoint) (+ (caddr nextpoint) vinc)))
  (command nextpoint)
;;; 4 12 4 11
  (setq vinc (* vstep 12))
  (setq dinc (* (* faceinc 12) -1))
  (setq xyspace (polar basepoint segmentangle (+ facedist dinc))) ; X&Y Components of distance from base ( angular ). 
  (setq nextpoint (list (car xyspace) (cadr xyspace) (+ (caddr nextpoint) vinc)))
  (command nextpoint)
;;;
  (setq vinc (* vstep 4))
  (setq nextpoint (list (car nextpoint) (cadr nextpoint) (+ (caddr nextpoint) vinc)))
  (command nextpoint)
;;;
  (setq vinc (* vstep 12))
  (setq dinc (* faceinc 12))
  (setq xyspace (polar basepoint segmentangle facedist)) ; X&Y Components of distance from base ( angular ). 
  (setq nextpoint (list (car xyspace) (cadr xyspace) (+ (caddr nextpoint) vinc)))
  (command nextpoint)

  )


;;; Now back to middle, above the origin... 
  (setq nextpoint (list (car basepoint) (cadr basepoint) (caddr nextpoint) ))
  (command nextpoint)
  (command "C")					; Close? Yep, Close...  
;;;  (command "")                                  ; No more points to define.. How do I close it?
  (princ)
) 

;;; ISO standard profile NUT pitch generation routine. (Internal Threads) - To subtract from material. 
;;;
(defun cnut (basepoint diameter pitch steppos numthreads / segments rads rinc id segmentangle faceinc facedist nextpoint startpoint nextpoint dinv vinc vstep xyspace loop) 
  (setvar "blipmode" 0)                         ; turn blipmode off
  (setvar "cmdecho" 0)                          ; turn cmdecho off
  (setq segments 32)		                ; 32 segments in this circle.
  (setq rads (* 3.141592653 2))                 ; PI x 2 ( Radians in a circle )
  (setq rinc (/ rads segments))                 ; Angle Increment in Radians = 1/Segment of a circle.
  (setq id (- diameter (* 0.75775 pitch)))       ; Internal Diameter = 7/8 of Thread Height ( ISO standard )
  (setq segmentangle (* rads (/ steppos pitch))); Segmentangle = 0 ( starting at origin... ) or twist more as progress...
  (setq faceinc (*  pitch (/ 1.732 segments)))  ; 60 degrees step per variation. 
  (setq facedist (/ diameter 2))	        ; initial distance of face from center = radius.

  (setq basepoint (list (car basepoint) (cadr basepoint) (- (+ (caddr basepoint) steppos) pitch))) ; Update base for step position and start one turn early. 
  (setq basepoint (polar basepoint segmentangle (- facedist (* pitch 0.8660)))) ; And offset most of the way... - PITCH.
  (setq facedist (* pitch 0.8660)) ; adjusted new face distance. 

  (setq startpoint (polar basepoint segmentangle facedist)) ; Set the last point to the same as the first point. 
  (command "3dpoly" basepoint startpoint)       ; Draw the bottom line of a vertical thread.
  (setq vinc 0.0)				; Vertical change in line.
  (setq dinc 0.0)				; Distance from basepoint change in line.
  (setq vstep (* 0.03125 pitch))
  (setq nextpoint startpoint)
;;; Now ready to start drawing vertical line.
;;; Four x rinc upwards

  
;;; Number of threads and re-calculate offset to remove first thread later.
;;;  this loops enough for all thread rises, plus 2 thread rises for the start and end.
  (setq loop (+ (/ length pitch) 2))
  (while (> loop 0) (setq loop (- loop 1))
  

  (setq vinc (* vstep 2))
  (setq nextpoint (list (car nextpoint) (cadr nextpoint) (+ (caddr nextpoint) vinc)))
  (command nextpoint)
;;; 4 12 4 11
  (setq vinc (* vstep 11))
  (setq dinc (* (* faceinc 11) -1))
  (setq xyspace (polar basepoint segmentangle (+ facedist dinc))) ; X&Y Components of distance from base ( angular ). 
  (setq nextpoint (list (car xyspace) (cadr xyspace) (+ (caddr nextpoint) vinc)))
  (command nextpoint)
;;;
  (setq vinc (* vstep 8))
  (setq nextpoint (list (car nextpoint) (cadr nextpoint) (+ (caddr nextpoint) vinc)))
  (command nextpoint)
;;;
  (setq vinc (* vstep 11))
  (setq dinc (* faceinc 11))
  (setq xyspace (polar basepoint segmentangle facedist)) ; X&Y Components of distance from base ( angular ). 
  (setq nextpoint (list (car xyspace) (cadr xyspace) (+ (caddr nextpoint) vinc)))
  (command nextpoint)

  )


;;; Now back to middle, above the origin... 
  (setq nextpoint (list (car basepoint) (cadr basepoint) (caddr nextpoint) ))
  (command nextpoint)
  (command "C")					; Close? Yep, Close...  
;;;  (command "")                                  ; No more points to define.. How do I close it?
  (princ)
) 



;;;
;;;       Interactive thread generation
;;;



(defun C:BOLT (/ olderr ocmd oblp basep od pitch length twistlen boltend boltface baxis halfthread allparts lastpart step nextstep allsteps stepangle) 
  (setq olderr  *error*
        *error* myerror)
  (setq ocmd (getvar "cmdecho"))
  (setq oblp (getvar "blipmode"))
  (setvar "cmdecho" 0)
  (princ "Bolt creation ( external thread ) - use NUT for internal threads. Creates ISO standard threads in METRIC ")
  (initget 1)                         ; Can't just press enter. 
  (setq basep (getpoint "\nCenter point: "))
  (initget 7)                         ; OD must not be null, zero or negative.
  (setq od (getdist "\nDiameter: "))
  (initget 7)                         ; pitch must not be null, zero or negative
  (setq pitch (getdist "\nPitch: "))
  (initget 7)                         ; length must not be null, zero or negative
  (setq length (getdist "\nLength of threads: "))

  (setq stepangle 0.0) ; Initial angle of rotation based on vertical steps ( affected by pitch )

  (setq halfthread (ssadd)) ; prepare set for first half.
  (setq allparts (ssadd))  ; Prepare set for two halfs.

;;; Draw the first half
  (repeat 17					; 16 segments to a half-circle = 17 faces. 
  (cbolt basep od pitch stepangle length)
  (setq lastpart (entlast))
  (ssadd lastpart halfthread)
  (setq stepangle (+ stepangle (/ pitch 32)))
  )

  (command "loft" halfthread "" "" )
  (setq lastpart (entlast))
  (ssadd lastpart allparts)

  (command "delete" halfthread "")


  (setq stepangle (- stepangle (/ pitch 32))) ; backstep a little.
  (setq halfthread (ssadd)) ; prepare set for second half.

;;; Draw the second half...
  (repeat 17
  (cbolt basep od pitch stepangle length)
  (setq lastpart (entlast))
  (ssadd lastpart halfthread)
  (setq stepangle (+ stepangle (/ pitch 32)))
  )

  (command "loft" halfthread "" "" )
  (setq lastpart (entlast))
  (ssadd lastpart allparts)
  (command "delete" halfthread "")
  (command "union" allparts "")		; And unify the threads... Now there's a thread tube = Thread ID.

;;; Create a mask of the original bolt diameter and intersect with the thread tube to remove excess threads.
  (command "circle" basep (/ od 2))  ; Mask off the 
  (setq lastpart (entlast))
  (ssadd lastpart allparts)
  (command "extrude" lastpart "" length)
  (setq lastpart (entlast))
  (ssadd lastpart allparts)
  (command "intersect" allparts "")
  (setq allparts (ssadd))  ; clear list of old parts.
  (setq lastpart (entlast))
  (ssadd lastpart allparts) ; And make the thread tube part of new "ALLPARTS"

;;; Now create the inside of the bolt. 
  (command "circle" basep (- (/ od 2) (* pitch 0.8660)))
  (setq lastpart (entlast))
  (ssadd lastpart allparts)
  (command "extrude" lastpart "" length)
  (setq lastpart (entlast))
  (ssadd lastpart allparts) ; And add the "inside" tube to the threads. ... ( remove this section of code for hollow threads at min depth )

;;; Now UNION the inside and the threads and the bolt shank is complete... Later modify for unthreaded section also. 
  (command "union" allparts "")  

;;; The bolt is now complete.

;;; This section dtaws a line down the axis if required... Also calculates the bolt end for shank inclusion.
  (setq boltend (list (car basep) (cadr basep) (+ (caddr basep) length)))
;;;  (boltaxis basep boltend)  ; Remove this line if you don't want an axis line... ( not normally needed )

  (setvar "cmdecho" ocmd)
  (setvar "blipmode" oblp)
  (setq *error* olderr)               ; Restore old *error* handler
  (princ)
)

(defun C:NUT (/ olderr ocmd oblp basep od pitch length twistlen boltend boltface baxis halfthread allparts lastpart step nextstep allsteps stepangle clearance) 
  (setq olderr  *error*
        *error* myerror)
  (setq ocmd (getvar "cmdecho"))
  (setq oblp (getvar "blipmode"))
  (setvar "cmdecho" 0)
  (princ "Nut creation ( external thread ) - use BOLT for external threads. Creates ISO standard threads in METRIC ")
  (initget 1)                         ; Can't just press enter. 
  (setq basep (getpoint "\nCenter point: "))
  (initget 7)                         ; OD must not be null, zero or negative.
  (setq od (getdist "\nDiameter: "))
  (initget 7)                         ; pitch must not be null, zero or negative
  (setq pitch (getdist "\nPitch: "))
  (initget 7)                         ; length must not be null, zero or negative
  (setq length (getdist "\nMinimum Length of threads: "))

;;; For NUTS, get a "clearance" amount to make them bigger.
  (initget 4)                         ; Diameter clearance must not be zero or negative
  (setq clearance (getdist "\nClearance between parts to allow for 3D printing shrinkage (0.0): "))
  (cond ((null clearance) (setq clearance 0.0))) 

  (setq clearance (* clearance 2))
  (setq od (+ od clearance ))



  (setq stepangle 0.0) ; Initial angle of rotation based on vertical steps ( affected by pitch )

  (setq halfthread (ssadd)) ; prepare set for first half.
  (setq allparts (ssadd))  ; Prepare set for two halfs.

;;; Draw the first half
  (repeat 17					; 16 segments to a half-circle = 17 faces. 
  (cnut basep od pitch stepangle length)
  (setq lastpart (entlast))
  (ssadd lastpart halfthread)
  (setq stepangle (+ stepangle (/ pitch 32)))
  )

  (command "loft" halfthread "" "" )
  (setq lastpart (entlast))
  (ssadd lastpart allparts)

  (command "delete" halfthread "")


  (setq stepangle (- stepangle (/ pitch 32))) ; backstep a little.
  (setq halfthread (ssadd)) ; prepare set for second half.

;;; Draw the second half...
  (repeat 17
  (cnut basep od pitch stepangle length)
  (setq lastpart (entlast))
  (ssadd lastpart halfthread)
  (setq stepangle (+ stepangle (/ pitch 32)))
  )

  (command "loft" halfthread "" "" )
  (setq lastpart (entlast))
  (ssadd lastpart allparts)
  (command "delete" halfthread "")
  (command "union" allparts "")		; And unify the threads... Now there's a thread tube = Thread ID.

;;; Create a mask of the original bolt diameter and intersect with the thread tube to remove excess threads.
;  (command "circle" basep (/ od 2))  ; Mask off the 
;  (setq lastpart (entlast))
;  (ssadd lastpart allparts)
;  (command "extrude" lastpart "" length)
;  (setq lastpart (entlast))
;  (ssadd lastpart allparts)
;  (command "intersect" allparts "")
  (setq allparts (ssadd))  ; clear list of old parts.
  (setq lastpart (entlast))
  (ssadd lastpart allparts) ; And make the thread tube part of new "ALLPARTS"

;;; Now create the inside of the nut. 
  (command "circle" basep (- (/ od 2) (* pitch 0.8660)))
  (setq lastpart (entlast))
  (ssadd lastpart allparts)
  (command "extrude" lastpart "" length)
  (setq lastpart (entlast))
  (ssadd lastpart allparts) ; And add the "inside" tube to the threads. ... ( remove this section of code for hollow threads at min depth )

;;; Now UNION the inside and the threads and the bolt shank is complete... Later modify for unthreaded section also. 
  (command "union" allparts "")  

;;; The nut is now complete.

;;; This section dtaws a line down the axis if required... Also calculates the bolt end for shank inclusion.
  (setq boltend (list (car basep) (cadr basep) (+ (caddr basep) length)))
;;;  (boltaxis basep boltend)  ; Remove this line if you don't want an axis line... ( not normally needed )

  (setvar "cmdecho" ocmd)
  (setvar "blipmode" oblp)
  (setq *error* olderr)               ; Restore old *error* handler
  (princ)
)



;;; --------------------------------------------------------------------------;

